{ pkgs ? import <nixpkgs> { } }:
let
  rust_overlay = import (builtins.fetchTarball "https://github.com/oxalica/rust-overlay/archive/master.tar.gz");
  pkgs = import <nixpkgs> { config.allowUnfree = true; overlays = [ rust_overlay ]; };
  rustVersion = "1.69.0";
  rust = pkgs.rust-bin.stable.${rustVersion}.default.override {
    extensions = [ "rust-src" ];
  };
in
pkgs.mkShell {
  buildInputs = [
    rust
  ] ++ (with pkgs; [
    pkgconfig
    rust-analyzer
    cmake
    fontconfig
    xorg.libX11
    xorg.libXi
    xorg.libXcursor
    xorg.libXrandr
    xorg.libXext
    xorg.libxshmfence
    xorg.libXxf86vm
    libxkbcommon
    libGL
    glslang
    vulkan-headers
    vulkan-loader
    vulkan-validation-layers
    vulkan-tools
  ]);

  shellHook = ''
    export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${
      with pkgs;
      lib.makeLibraryPath [ vulkan-loader libGL xorg.libXrandr xorg.libXcursor xorg.libX11 xorg.libXi libxkbcommon ]
    }"
  '';
  
  RUST_BACKTRACE = 0;
}
