use winit::window::Window;
use imgui_winit_support::{
    HiDpiMode,
    WinitPlatform,
};
use imgui_wgpu::{
    Renderer,
    RendererConfig,
};
use imgui::{
    Context,
    FontConfig,
    FontSource,
    ConfigFlags,
    MouseCursor,
};
use wgpu::{
    Queue,
    Device,
    SurfaceConfiguration,
};

pub struct Imgui {
    pub context: Context,
    pub renderer: Renderer,
    pub platform: WinitPlatform,
    pub last_cursor: Option<MouseCursor>,
}

impl Imgui {
    pub fn new(window: &Window, device: &Device, queue: &Queue, config: &SurfaceConfiguration) -> Self {
        let mut context = Context::create();
        let mut platform = WinitPlatform::init(&mut context);
        let scale_factor = window.scale_factor();

        platform.attach_window(
            context.io_mut(),
            &window,
            HiDpiMode::Default,
        );
        context.set_ini_filename(None);

        context.io_mut().font_global_scale = (1.0/scale_factor) as f32;
        context.io_mut().config_flags |= ConfigFlags::DOCKING_ENABLE;

        let font_config = FontConfig {
            oversample_h: 1,
            pixel_snap_h: true,
            size_pixels: (13.0 * scale_factor) as f32,
            ..Default::default()
        };
        let font = FontSource::DefaultFontData {
            config: Some(font_config)
        };
        context.fonts().add_font(&[font]);

        let renderer_config = RendererConfig {
            texture_format: config.format,
            ..Default::default()
        };
        let renderer = Renderer::new(&mut context, &device, &queue, renderer_config);
        let last_cursor = None;

        Imgui {
            context,
            platform,
            renderer,
            last_cursor,
        }
    }
}
