mod gui;
mod state;

use std::time::Instant;

use gui::Imgui;
use state::State;

use pollster::block_on;

use winit::event::Event;
use winit::event::WindowEvent;
use winit::event_loop::EventLoop;
use winit::window::WindowBuilder;


fn main() {
    let event_loop = EventLoop::new();

    let window = WindowBuilder::new()
        .with_inner_size(winit::dpi::LogicalSize::new(800.0, 600.0))
        .with_title("A fantastic window!")
        .build(&event_loop)
        .expect("FFFFFFFFFFFf couldn't create window");
    
    let mut state = block_on(State::new(window));
    let mut imgui = Imgui::new(&state.window, &state.device, &state.queue, &state.config);

    let mut inst = Instant::now();

    event_loop.run(move |event, _, control_flow| {
        //control_flow.set_wait();

        match event {
            Event::WindowEvent {
                ref event,
                window_id,
            } if window_id == state.window().id() => if !state.input(event) {
                match event {
                    WindowEvent::Resized(size) => state.resize(*size),
                    WindowEvent::ScaleFactorChanged {
                        new_inner_size, ..
                    } => state.resize(**new_inner_size),
                    WindowEvent::CloseRequested => control_flow.set_exit(),
                    _ => (),
                }
            }
            Event::MainEventsCleared => state.window().request_redraw(),
            Event::RedrawRequested(_) => { 
                let now = Instant::now();
                let delta = (now-inst).as_secs_f64();

                if delta >= 0.0166666666666 {
                    println!("Framerate: {:?} fps", 1.0/delta);

                    //state.render(&mut imgui);
                    let now2 = Instant::now();
                    println!("Renderer took {:?}s", (now2-inst).as_secs_f64());
                    inst = now;
                }
            },
            _ => (),
        }

        imgui.platform.handle_event(imgui.context.io_mut(), &state.window(), &event);
    });
}
